#!/bin/bash

#BSUB -W 24:00
#BSUB -o logs/euler_logs/mergeMosaic.%J.%I.txt
#BSUB -e logs/euler_logs/mergeMosaic.%J.%I.txt
#BSUB -R "rusage[mem=12000]"
#BSUB -n 18
#BSUB -J "mergeMosaic[1]"
#BSUB -N
#BSUB -u yujiang@ethz.ch

# --- SET DIRECTORY ---
# parent_dir contains tile results for all variables, predictions for all years
# e.g. output/predict_tiles(parent_dir)/    mean{var}/mean(subdir)

parent_dir=output/predict_tiles
vars="mean max"
ch_years="2020"  # 2017, 2018, 2019 ok

# select a subdirectory, e.g. mean, std
subdirs=(mean)
index=$((LSB_JOBINDEX - 1))
subdir=${subdirs[$index]}
nodata_value=nan   #TODO set to: 'nan' (if dtype float) or e.g. 255 (if dtype byte)

echo "parent_dir:" $parent_dir
echo "vas": $vars
echo "subdir:" $subdir
# -------------

cd ${parent_dir}

# loop through all variables
for var in $vars; do
    cd ${var}
        echo "Process variable $var"
        # loop trough all years
        for ch_year in $ch_years; do
            echo "Merge tiles for year $ch_year"

            # warp each tile to a vrt file in WGS84
            for file in ${subdir}/${ch_year}*.tif; do
                out_file=${file/.tif/.vrt}
                echo warping to ${out_file}
                gdalwarp -overwrite -t_srs "EPSG:4326" -co "TILED=YES" -of VRT -srcnodata ${nodata_value} -dstnodata ${nodata_value} ${file} ${out_file}
            done

            # create a big mosaic using all WGS84 vrts
            echo create mosaic_ch_year_preds.vrt ...
            gdalbuildvrt mosaic_${ch_year}_${subdir}.vrt ${subdir}/${ch_year}*.vrt

            # create pyramid
            echo create pyramid ...
            gdaladdo -r average mosaic_${ch_year}_${subdir}.vrt 2 4 8 16 32 --config COMPRESS_OVERVIEW DEFLATE --config PREDICTOR_OVERVIEW 2
        done

    cd ../
done





