"""

loss function

"""

import torch
import numpy as np
from src.utils.utils_train import nanmean


def negative_log_likelihood(mean, log_var, labels):
    """
    Negative log likelihood of the data. Adding weight decay yields the negative log posterior. Note that we're
    interpreting all labels of a given pixel as individual training samples, corresponding to an assumed multivariate
    Gaussian with diagonal covariance matrix (i.e. independent label noise over tasks). Also, we only consider
    locations where labels != nan.
    """
    assert mean.shape == log_var.shape == labels.shape
    precision = torch.exp(-log_var)
    diff = mean - labels
    # we set any nan's to zero to prevent nan gradients in the backward pass. This also stops gradient flow through
    # the locations where labels are nan, but we don't need gradients for these locations anyway. Further info:
    # https://github.com/pytorch/pytorch/issues/15506, https://github.com/pytorch/pytorch/issues/15131
    diff[diff.isnan()] = 0.
    losses = precision * diff**2 + log_var
    return losses

def mseloss(preds, labels):
    """
    compute standard mse for regression
    """
    diff = preds - labels
    diff[diff.isnan()] = 0.
    losses = diff**2

    return losses

'''
EDL loss
based on https://github.com/Dariusrussellkish/evidential-deep-learning/blob/pytorch_implementation/evidential_deep_learning/pytorch/losses/continous.py
'''
def NIG_NLL(y: torch.Tensor,
            gamma: torch.Tensor,
            nu: torch.Tensor,
            alpha: torch.Tensor,
            beta: torch.Tensor, reduction='mean'):
    inter = 2 * beta * (1 + nu)

    nll = 0.5 * (np.pi / nu).log() \
          - alpha * inter.log() \
          + (alpha + 0.5) * (nu * (y - gamma) ** 2 + inter).log() \
          + torch.lgamma(alpha) \
          - torch.lgamma(alpha + 0.5)
    return nll


def NIG_Reg(y, gamma, nu, alpha, reduction='mean'):
    error = (y - gamma).abs()
    evidence = 2. * nu + alpha
    return error * evidence
